from websockify.websocketproxy import WebSocketProxy
import time

LISTEN_HOST, LISTEN_PORT = "127.0.0.1", 6761
TARGET_HOST, TARGET_PORT = "127.0.0.1", 5761

def start_websocket_proxy():
    server = WebSocketProxy(
        listen_host=LISTEN_HOST,
        listen_port=LISTEN_PORT,
        target_host=TARGET_HOST,
        target_port=TARGET_PORT,
        daemon=False,
        verbose=True,
    )
    server.start_server()
def main():
    print(f"WebSocket proxy started on ws://{LISTEN_HOST}:{LISTEN_PORT} forwarding to {TARGET_HOST}:{TARGET_PORT}")
    start_websocket_proxy()
if __name__ == "__main__":
    main()

    

    




from sitl.simulator import L2F
from sitl.gamepad import Gamepad
import asyncio
import os
import time
import argparse


async def main():
    args = argparse.ArgumentParser(description="Run the L2F simulator with a gamepad.")
    args.add_argument("gamepad_mapping", type=str, help="Path to the gamepad mapping JSON file.")
    args = args.parse_args()
    time.sleep(1)
    simulator = L2F()
    gamepad = Gamepad(args.gamepad_mapping, simulator.set_rc_channels)

    await asyncio.gather(simulator.run(), gamepad.run())
def sync_main():
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    loop.run_until_complete(main())


if __name__ == "__main__":
    sync_main()


import websockets
import l2f
from betaflight import Simulator
import asyncio
import numpy as np
import json
import time
from scipy.spatial.transform import Rotation as R


crazyflie_from_betaflight_motors = [0, 3, 1, 2]

class L2F(Simulator):
    def __init__(self, UI_SERVER="ws://localhost:13337/backend", *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.UI_SERVER = UI_SERVER
        self.device = l2f.Device()
        self.rng = l2f.Rng()
        self.env = l2f.Environment()
        self.ui = l2f.UI()
        self.params = l2f.Parameters()
        self.state = l2f.State()
        self.next_state = l2f.State()
        l2f.initialize_rng(self.device, self.rng, 0)
        l2f.initialize_environment(self.device, self.env)
        l2f.sample_initial_parameters(self.device, self.env, self.params, self.rng)
        l2f.initial_state(self.device, self.env, self.params, self.state)
        self.previous_time = None
        self.simulation_dts = []
    async def step(self, rpms):
        simulation_dt = time.time() - self.previous_time
        self.previous_time = time.time()
        self.simulation_dts.append(simulation_dt)
        self.simulation_dts = self.simulation_dts[-100:]

        parameters_string = l2f.parameters_to_json(self.device, self.env, self.params)
        parameters = json.loads(parameters_string)
        parameters["integration"]["dt"] = simulation_dt
        l2f.parameters_from_json(self.device, self.env, json.dumps(parameters), self.params)
        
        action = np.array(rpms)[crazyflie_from_betaflight_motors] * 2 - 1
        dts = l2f.step(self.device, self.env, self.params, self.state, action, self.next_state, self.rng)
        acceleration = (self.next_state.linear_velocity - self.state.linear_velocity) / simulation_dt
        r = R.from_quat([*self.state.orientation[1:], self.state.orientation[0]])
        R_wb = r.as_matrix()
        accelerometer = R_wb.T @ (acceleration - np.array([0, 0, -9.81], dtype=np.float64))
        self.state.assign(self.next_state)
        if self.state.position[2] <= -0.05:
            self.state.position[2] = 0
            self.state.linear_velocity[:] = 0
            self.state.orientation = [1, 0, 0, 0]
            self.state.angular_velocity[:] = 0
        if np.linalg.norm(self.state.position) > 2:
            self.state.position[:] = 0
            self.state.linear_velocity[:] = 0
            self.state.orientation = [1, 0, 0, 0]
            self.state.angular_velocity[:] = 0
        state_action_message = l2f.set_state_action_message(self.device, self.env, self.params, self.ui, self.state, action)
        await self.websocket.send(state_action_message)
        # print(f"RPMs: {rpms} dt: {np.mean(self.simulation_dts):.4f} s, action: {action[0].tolist()}")
        return self.state.position, self.state.orientation, self.state.linear_velocity, self.state.angular_velocity, accelerometer, 101325

    async def run(self):
        self.websocket = await websockets.connect(self.UI_SERVER)
        handshake = json.loads(await self.websocket.recv(self.UI_SERVER))
        assert(handshake["channel"] == "handshake")
        namespace = handshake["data"]["namespace"]
        self.ui.ns = namespace
        ui_message = l2f.set_ui_message(self.device, self.env, self.ui)
        parameters_message = l2f.set_parameters_message(self.device, self.env, self.params, self.ui)
        await self.websocket.send(ui_message)
        await self.websocket.send(parameters_message)
        self.previous_time = time.time()
        await super().run()


import pygame
from copy import copy
import numpy as np
import asyncio
import json


betaflight_order = ["roll", "pitch", "throttle", "yaw", "arm"] # AETR

pygame.init()
pygame.joystick.init()
if pygame.joystick.get_count() > 0:
    joystick = pygame.joystick.Joystick(0)
    joystick.init()
else:
    joystick = None


class Gamepad:
    def __init__(self, mapping, callback):
        with open(mapping, 'r') as f:
            self.gamepad_mapping = json.load(f)
        self.callback = callback

    async def run(self):
        while True:
            await asyncio.sleep(0.01)
            pygame.event.pump()
            if joystick is None:
                continue
            axes = [joystick.get_axis(i) for i in range(joystick.get_numaxes())]
            buttons = [joystick.get_button(i) for i in range(joystick.get_numbuttons())]
            rc = []
            for key in betaflight_order:
                cfg = self.gamepad_mapping[key]
                if "axis" in cfg:
                    idx = cfg["axis"]
                    v = axes[idx] if idx < len(axes) else 0.0
                else:
                    idx = cfg["button"]
                    v = 1.0 if idx < len(buttons) and buttons[idx] else 0.0
                if cfg.get("invert", False):
                    v = -v
                rc.append(int((v + 1) * 500 + 1000))
            self.callback(rc)

def main():
    initial_axes = None
    while True:
        pygame.event.pump()
        axes = [joystick.get_axis(i) for i in range(joystick.get_numaxes())]
        if initial_axes is None:
            initial_axes = copy(axes)
        if sum(np.abs(np.array(axes) - initial_axes) > 0.25) == 1:
            diffs = np.abs(np.array(axes) - initial_axes)
            print("Axis: ", np.argmax(diffs), "Value: ", axes[np.argmax(diffs)])
        buttons = [joystick.get_button(i) for i in range(joystick.get_numbuttons())]
        for i, button in enumerate(buttons):
            if button:
                print(f"Button {i} pressed")